# -*- coding:utf-8 -*-
# rb2html
# Copyright (c) 2001, 2004, 2006 HORIKAWA Hisashi. All rights reserved.
#     http://www.nslabs.jp/
#     mailto:vzw00011@nifty.ne.jp

require 'rb2html/pattern_lexer'

module Rb2HTML
  class PythonLexer < PatternLexer

    PYTHON_PATTERNS = [
    [/([0-9]*\.[0-9]+|[0-9]+\.)([eE][+-]?[0-9]+)?j?/, :numeric],
    [/0[xX][0-9A-Fa-f]+/, :numeric],

    [/[0-9]+[Llj]?/, :numeric],

    [/([ruRU]|[uU][rR])?'''(\\.|[^'])*'''/, :string],
    [/([ruRU]|[uU][rR])?"""(\\.|[^"])*"""/, :string],

    # TODO: raw stringの対応
    [/([ruRU]|[uU][rR])?'(\\.|[^'])*'/, :string],
    [/([ruRU]|[uU][rR])?"(\\.|[^"])*"/, :string],

    [/[a-zA-Z_][a-zA-Z0-9_]*/, :ident],
    [%r(\+|-|\*|\*\*|/|//|%|<<|>>|&|\||\^|~|<|>|<=|>=|==|!=), :operator],
    
    # Pythonの代入は文
    [%r(=|\+=|-=|\*=|/=|//=|%=|&=|\|=|\^=|>>=|<<=|\*\*=), :assignment],
    
    [/#.*/, :comment],
    [/\r?\n|\r/, :eol],
    [/\s+/, :space],
    ]

    KEYWORDS = %w(and       del       for       is        raise    
        assert    elif      from      lambda    return   
        break     else      global    not       try      
        class     except    if        or        while    
        continue  exec      import    pass      yield    
        def       finally   in        print
        as None)

    def parse(source_, &block)
      super(source_)
      while !@scanner.eos?
        matched = match(PYTHON_PATTERNS) {|sym, r|
          #puts "text = #{r}, sym = #{sym}, pat = #{pat}"
          if sym == :ident
            if KEYWORDS.include?(r)
              flush_token Token.new(:keyword, r)
            else
              flush_token Token.new(sym, r)
            end
          else
            flush_token Token.new(sym, r)
          end
        }

        if !matched
          @cur.symbol = :other
          @cur.text << @scanner.getch
        end
      end
      return @parsed
    end
  end
end

